/**********************************************************************
 * Copyright 2013 Georg Rudoy <0xd34df00d@gmail.com>
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "operation.h"
#include <algorithm>

namespace Laretz
{
	Operation::Operation ()
	: m_type (OpType::Modify)
	{
	}

	Operation::Operation (OpType op, const std::vector<Item>& items)
	: m_type (op)
	, m_items (items)
	{
	}

	OpType Operation::getType () const
	{
		return m_type;
	}

	void Operation::setType (OpType type)
	{
		m_type = type;
	}

	const std::vector<Item>& Operation::getItems () const
	{
		return m_items;
	}

	std::vector<Item>& Operation::getItems()
	{
		return m_items;
	}

	void Operation::setItems (const std::vector<Item>& item)
	{
		m_items = item;
	}

	bool Operation::empty () const
	{
		return m_items.empty ();
	}

	bool Operation::contains (const std::string& id) const
	{
		return std::find_if (m_items.begin (), m_items.end (),
				[&id] (const Item& item) { return item.getId () == id; }) != m_items.end ();
	}

	Operation& Operation::operator+= (const Item& item)
	{
		const auto pos = std::find_if (m_items.begin (), m_items.end (),
				[&item] (const Item& it) { return it.getId () == item.getId (); });
		if (pos == m_items.end ())
			m_items.push_back (item);
		else
			*pos += item;

		return *this;
	}

	bool Operation::operator-= (const Item& item)
	{
		const auto pos = std::find_if (m_items.begin (), m_items.end (),
				[&item] (const Item& it) { return it.getId () == item.getId (); });

		const bool del = pos != m_items.end ();
		if (del)
			m_items.erase (pos);

		return del;
	}

	Operation& Operation::operator+= (const Operation& op)
	{
		for (const auto& item : op.getItems ())
			*this += item;

		return *this;
	}
}
